@@ -330,11 +330,11 @@ void HtmlPage::endString() {
   // throw away zero-length strings -- they don't have valid xMin/xMax
   // values, and they're useless anyway
     
-  if (curStr->len == 0) {
+   if (curStr->len == 0) {
     delete curStr;
-    curStr = NULL;
+     curStr = NULL;
     return;
-  }
+   }
 
   curStr->endString();
 
@@ -460,17 +460,19 @@ void HtmlPage::coalesce() {
         !(str2->len == 1 && str2->htext->getCString()[0] == ' ')
         &&
         !(str1->htext->getCString()[str1->len-1] == ' ')
+        &&
+        !(str1->htext->getLength() >= str1->len+1 && str1->htext->getCString()[str1->len+1] == ' ')
        ) 
     {
         diff = str2->xMax - str1->xMin;
 
         n = str1->len + str2->len;
 
-        /*
+        
         if ((addSpace = horSpace > 0.1 * space)) {
             ++n;
         }
-         */
+         
               
         str1->size = (n + 15) & ~15;
         str1->text = (Unicode *)grealloc(str1->text,
@@ -478,72 +480,83 @@ void HtmlPage::coalesce() {
         str1->xRight = (double *)grealloc(str1->xRight,
 					str1->size * sizeof(double));
         
-        /*if (addSpace) {
+        if (addSpace) {
             str1->text[str1->len] = 0x20;
             str1->htext->append(" ");
             str1->htext2->append(" ");
             str1->xRight[str1->len] = str2->xMin;
             ++str1->len;
             ++str1->strSize;
-        }*/
-      
-        str1->htext2->append(str2->htext2);
-
-        XmlLink *hlink1 = str1->getLink();
-        XmlLink *hlink2 = str2->getLink();
-
-        for (i = 0; i < str2->len; ++i) {
-            str1->text[str1->len] = str2->text[i];
-            str1->xRight[str1->len] = str2->xRight[i];
-            ++str1->len;
-        }
-
-        if( !hlink1 || !hlink2 || !hlink1->isEqualDest(*hlink2) ) {
             
-            if(hlink1 != NULL ){
-                //str1->htext->append("\"]");
-            }
-            if(hlink2 != NULL ) {
-                GString *ls = hlink2->getLinkStart();
-                str1->htext->append(ls);
-                delete ls;
-            }
+            str1->xMin = curX; str1->yMin = curY; 
+	        str1 = str2;
+	        curX = str1->xMin; curY = str1->yMin;
+	        hfont1 = hfont2;
+	
+	        if( str1->getLink() != NULL ) {
+	            GString *ls = str1->getLink()->getLinkStart();
+	            str1->htext->insert(0, ls);
+	            delete ls;
+	        }
+        }else{
+	        str1->htext2->append(str2->htext2);
+	
+	        XmlLink *hlink1 = str1->getLink();
+	        XmlLink *hlink2 = str2->getLink();
+	
+	        for (i = 0; i < str2->len; ++i) {
+	            str1->text[str1->len] = str2->text[i];
+	            str1->xRight[str1->len] = str2->xRight[i];
+	            ++str1->len;
+	        }
+	
+	        if( !hlink1 || !hlink2 || !hlink1->isEqualDest(*hlink2) ) {
+	            
+	            if(hlink1 != NULL ){
+	                //str1->htext->append("\"]");
+	            }
+	            if(hlink2 != NULL ) {
+	                GString *ls = hlink2->getLinkStart();
+	                str1->htext->append(ls);
+	                delete ls;
+	            }
+	        }
+	
+	        str1->htext->append(str2->htext);
+	        sSize = str1->htext2->getLength();      
+	        pxSize = xoutRoundLower(hfont1->getSize()/scale);
+	        strSize = (pxSize*(sSize-2));   
+	        cspace = (diff / strSize);//(strSize-pxSize));
+	        // we check if the fonts are the same and create a new font to ajust the text
+	        //      double diff = str2->xMin - str1->xMin;
+	        //      printf("%s\n",str1->htext2->getCString());
+	        // str1 now contains href for link of str2 (if it is defined)
+	        str1->link = str2->link; 
+	
+	        //XmlFont *newfnt = new XmlFont(*hfont1);
+	        //newfnt->setCharSpace(cspace);
+	        //newfnt->setLineSize(curLineSize);
+	        //str1->fontpos = fonts->AddFont(*newfnt);
+	        //delete newfnt;
+	        hfont1 = getFont(str1);
+	        // we have to reget hfont2 because it's location could have
+	        // changed on resize  GStri;ng *iStr=GString::fromInt(i);
+	        hfont2 = getFont(str2); 
+	
+	        hfont1 = hfont2;
+	
+	        if (str2->xMax > str1->xMax) {
+	            str1->xMax = str2->xMax;
+	        }
+	        
+	        if (str2->yMax > str1->yMax) {
+	            str1->yMax = str2->yMax;
+	        }
+	
+	        str1->yxNext = str2->yxNext;
+	
+	        delete str2;
         }
-
-        str1->htext->append(str2->htext);
-        sSize = str1->htext2->getLength();      
-        pxSize = xoutRoundLower(hfont1->getSize()/scale);
-        strSize = (pxSize*(sSize-2));   
-        cspace = (diff / strSize);//(strSize-pxSize));
-        // we check if the fonts are the same and create a new font to ajust the text
-        //      double diff = str2->xMin - str1->xMin;
-        //      printf("%s\n",str1->htext2->getCString());
-        // str1 now contains href for link of str2 (if it is defined)
-        str1->link = str2->link; 
-
-        //XmlFont *newfnt = new XmlFont(*hfont1);
-        //newfnt->setCharSpace(cspace);
-        //newfnt->setLineSize(curLineSize);
-        //str1->fontpos = fonts->AddFont(*newfnt);
-        //delete newfnt;
-        hfont1 = getFont(str1);
-        // we have to reget hfont2 because it's location could have
-        // changed on resize  GStri;ng *iStr=GString::fromInt(i);
-        hfont2 = getFont(str2); 
-
-        hfont1 = hfont2;
-
-        if (str2->xMax > str1->xMax) {
-            str1->xMax = str2->xMax;
-        }
-        
-        if (str2->yMax > str1->yMax) {
-            str1->yMax = str2->yMax;
-        }
-
-        str1->yxNext = str2->yxNext;
-
-        delete str2;
     } else { 
 
         //printf("startX = %f, endX = %f, diff = %f, fontsize = %d, pxSize = %f, stringSize = %d, cspace = %f, strSize = %f\n",str1->xMin,str1->xMax,diff,hfont1->getSize(),pxSize,sSize,cspace,strSize);
@@ -817,7 +830,7 @@ ImgOutputDev::ImgOutputDev(char *fileName, char *title,
   pages = new HtmlPage(rawOrder, textAsJSON, compressData, extension);
   
   glMetaVars = new GList();
-  glMetaVars->append(new HtmlMetaVar("generator", "pdf2json 0.61"));  
+  glMetaVars->append(new HtmlMetaVar("generator", "pdf2json 0.68"));  
   if( author ) glMetaVars->append(new HtmlMetaVar("author", author));  
   if( keywords ) glMetaVars->append(new HtmlMetaVar("keywords", keywords));  
   if( date ) glMetaVars->append(new HtmlMetaVar("date", date));  
@@ -838,7 +851,7 @@ ImgOutputDev::ImgOutputDev(char *fileName, char *title,
       //else if (textAsJSON) right->append(".js");
       
       if(this->split>0 && this->hasValidSplitFileName()){
-        this->setSplitFileName(10,false);
+        this->setSplitFileName(this->split,false);
       }else if (!(page=fopen(right->getCString(),"w"))){
           delete right;
           error(-1, "Couldn't open html file '%s'", right->getCString());
@@ -1009,7 +1022,12 @@ void ImgOutputDev::endPage() {
   if(this->split>0 && (pageNum % this->split) == 0 && this->hasValidSplitFileName()){
       if(textAsJSON){fputs("]",page);}else{fputs("</pdf2xml>\n",page);}
       this->passedFirstPage = gFalse;
-      this->setSplitFileName(pageNum + this->split,true);
+      //this->setSplitFileName(pageNum + this->split,true);
+      if(this->split>1) {
+         this->setSplitFileName(pageNum + this->split,true);
+      } else {
+         this->setSplitFileName(pageNum,true);
+      }
       if(textAsJSON){fputs("[",page);}else{fputs("<pdf2xml>\n",page);  }
       //printf("PROCESS: %s",tmp->getCString());
       //printf("mod %d",(this->split-(this->split/pageNum)*pageNum)); 
@@ -1269,8 +1287,11 @@ GString* ImgOutputDev::getLinkDest(Link *link,Catalog* catalog){
           LinkDest *dest=NULL;
           if (ha->getDest()==NULL) 
               dest=catalog->findDest(ha->getNamedDest());
-          else 
-              dest=ha->getDest()->copy();
+          else if (ha->getNamedDest()!=NULL)
+          	  dest=catalog->findDest(ha->getNamedDest());
+          else
+	      	  dest=ha->getDest()->copy();
+	      
           if (dest){ 
               if (dest->isPageRef()){
                   Ref pageref=dest->getPageRef();
@@ -1322,7 +1343,8 @@ GString* ImgOutputDev::getLinkDest(Link *link,Catalog* catalog){
 	  { 
           LinkURI *ha=(LinkURI *) link->getAction();
           //GString* file=new GString(ha->getURI()->getCString());
-          GString *file=new GString("actionURI:");
+          GString *file=new GString("actionURI");
+          file->append("(");file->append(ha->getURI()->getCString());file->append("):");
           //file->append(ha->getURI()->getCString());
           // printf("uri : %s\n",file->getCString());
           return file;
